@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SweepContent.pm,v 1.17.2.15 2004/05/27 20:37:53 jkf Exp $
+#   $Id: SweepContent.pm 4531 2008-08-31 10:17:21Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ use DirHandle;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.17.2.15 $, 10;
+$VERSION = substr q$Revision: 4531 $, 10;
 
 # Attributes are
 #
@@ -79,6 +79,7 @@ sub ScanBatch {
   $counter = 0;
   $stripcounter = 0; # No. of messages we need to strip HTML from
 
+  $0 = 'MailScanner: dangerous content scanning';
   while(($id, $message) = each %{$batch->{messages}}) {
     next if $message->{deleted};
 
@@ -95,9 +96,9 @@ sub ScanBatch {
     # next message) probably won't have a filename.
     if ((! MailScanner::Config::Value('allowpartial', $message)) &&
         FindPartialMessage($message, $ent)) {
-      $message->{otherreports}{""} .=
-        MailScanner::Config::LanguageValue($message, 'partialmessage') . "\n";
-      $message->{othertypes}{""} .= "e";
+      #$message->{otherreports}{""} .=
+      #  MailScanner::Config::LanguageValue($message, 'partialmessage') . "\n";
+      #$message->{othertypes}{""} .= "e";
       #$message->{otherinfected}++;
       $counter++;
       # Force replacement of the last bit of the MIME message
@@ -123,13 +124,15 @@ sub ScanBatch {
     # Go through the attachments and remove any that are bigger than this
     # user/domain/site is allowed.
     my $maxmessagesize = MailScanner::Config::Value('maxmessagesize', $message);
+    $message->{maxmessagesize} = $maxmessagesize; # Store it for reporting
     if ($maxmessagesize>0 && $message->{size}>$maxmessagesize) {
       MailScanner::Log::WarnLog("Content Checks: Message %s is bigger than " .
                                 "%d bytes", $message->{id}, $maxmessagesize);
       $message->{otherreports}{""} .=
-        MailScanner::Config::LanguageValue($message, 'toobig') . "\n";
-      $message->{othertypes}{""} .= "e";
-      $message->{otherinfected}++;
+        MailScanner::Config::LanguageValue($message, 'toobig') . ": " .
+                                           $message->{size} . " bytes\n";
+      $message->{othertypes}{""} .= "s";
+      $message->{sizeinfected}++;
       $counter++;
     }
 
@@ -143,6 +146,7 @@ sub ScanBatch {
     my $formvalue   = MailScanner::Config::Value('allowformtags',   $message);
     my $scriptvalue = MailScanner::Config::Value('allowscripttags', $message);
     my $webbugvalue = MailScanner::Config::Value('allowwebbugtags', $message);
+    my $phishingvalue = MailScanner::Config::Value('findphishing',  $message);
     my $allowiframes = 0;
     my $allowobjects = 0;
     my $allowforms   = 0;
@@ -153,12 +157,14 @@ sub ScanBatch {
     my $convertforms   = 0;
     my $convertscripts = 0;
     my $convertwebbugs = 0;
+    my $allowphishing = 0;
     # Allow the tags only if everyone allows them
     $allowiframes = 1 if $iframevalue =~ /^[1\s]+$/;
     $allowobjects = 1 if $objectvalue =~ /^[1\s]+$/;
     $allowforms   = 1 if $formvalue   =~ /^[1\s]+$/;
     $allowscripts = 1 if $scriptvalue =~ /^[1\s]+$/;
     $allowwebbugs = 1 if $webbugvalue =~ /^[1\s]+$/;
+    $allowphishing =1 if $phishingvalue !~ /1/;
     # Convert the tags if no-one blocks them and someone converts them
     $convertiframes = 1 if $iframevalue !~ /0/ && $iframevalue =~ /convert/i;
     $convertobjects = 1 if $objectvalue !~ /0/ && $objectvalue =~ /convert/i;
@@ -170,10 +176,12 @@ sub ScanBatch {
     #print STDERR "WebBugvalue = $webbugvalue\n";
     #print STDERR "Allowforms = $allowforms and convertforms = $convertforms\n";
 
+    #print STDERR "Allowphishing = $allowphishing\n";
+
     # Shortcut the check completely if they want to allow everything
     # and are not converting nasty tags to text
     if (!($allowiframes && $allowforms && $allowscripts && $allowobjects &&
-          !$stripdangerous) &&
+          $allowphishing && !$stripdangerous) &&
         FindHTMLExploits($message, $id, $ent,
                          $allowiframes, $allowobjects, $allowforms,
                          $allowscripts, $allowwebbugs,
@@ -272,11 +280,12 @@ sub CheckAttachmentSizes {
   my($message, $id) = @_;
 
   my($BaseDir, $basefh, $safename, $maxsize, $attachsize, $tnefname);
-  my($unsafename, $counter);
+  my($unsafename, $counter, $minsize, $attachentity);
 
   # Read the configuration setting, value<=0 implies setting not in use.
   $maxsize = MailScanner::Config::Value('maxattachmentsize', $message);
-  return 0 unless $maxsize>=0;
+  $minsize = MailScanner::Config::Value('minattachmentsize', $message);
+  return 0 if $maxsize<0 && $minsize<0;
   $tnefname = $message->{entity2file}{$message->{tnefentity}};
 
   # Get into the directory containing all the attachments
@@ -293,15 +302,33 @@ sub CheckAttachmentSizes {
 
     # "Safe" attachment filename is in $safename, this is what we stat
     $attachsize = -s "$BaseDir/$safename";
-    if ($attachsize > $maxsize) {
-      $unsafename = $message->{safefile2file}{$safename} || $tnefname;
-      MailScanner::Log::InfoLog("Attachment size check: %s > %s (%s)",
-                                $attachsize, $maxsize, $unsafename);
+    $unsafename = $message->{safefile2file}{$safename} || $tnefname;
+    $attachentity = $message->{file2entity}{$unsafename};
+    next unless $attachentity; # Only check attachment, not contents of zips
+    next if $safename =~ /^msg[-\d]+\.(txt|html)$/;
+    #print STDERR "\nSafename = $safename\n";
+    #print STDERR "Attachsize=$attachsize\nMin=$minsize\nMax=$maxsize\n";
+    if ($maxsize>=0 && $attachsize > $maxsize) {
+      #print STDERR "$safename is too big $attachsize > $maxsize\n";
+      #print STDERR "Unsafename is $unsafename\n";
+      MailScanner::Log::NoticeLog("Attachment size check: %s > %s (%s) in %s",
+                                $attachsize, $maxsize, $unsafename, $id);
       $message->{otherreports}{$safename} .=
-       MailScanner::Config::LanguageValue($message,'attachmenttoolarge') . "\n";
-      $message->{othertypes}{$safename}   .= "e";
+       MailScanner::Config::LanguageValue($message,'attachmenttoolarge') .
+         ": $attachsize bytes\n";
+      $message->{othertypes}{$safename}   .= "s";
       $counter++;
-      $message->{otherinfected}++;
+      $message->{sizeinfected}++;
+    }
+    if ($minsize>=0 && $attachsize < $minsize) {
+      #print STDERR "Attachment is too small\n";
+      MailScanner::Log::NoticeLog("Attachment size check: %s < %s (%s) in %s",
+                                $attachsize, $minsize, $unsafename, $id);
+      $message->{otherreports}{$safename} .=
+       MailScanner::Config::LanguageValue($message,'attachmenttoosmall') . "\n";
+      $message->{othertypes}{$safename}   .= "s";
+      $counter++;
+      $message->{sizeinfected}++;
     }
   }
 
@@ -327,8 +354,28 @@ sub FindPartialMessage {
   my $type = $entity->head->mime_attr('content-type');
   if ($type && $type =~ /message\/partial/i) {
     #print STDERR "Found partial message at entity $entity\n";
+    # Found one, so work out where it's stored and quarantine it
+    my $body = $entity->bodyhandle;
+    my $bodypath;
+    if ($body) {
+      $bodypath = $entity->bodyhandle->path;
+    } elsif ($entity->parts) {
+      my $part = $entity->parts(0);
+      $bodypath = $part->bodyhandle->path if $part && $part->bodyhandle;
+    } else {
+      $bodypath = "";
+    }
+    $bodypath =~ s/^.*\///; # Just want the filename
+    if ($bodypath) {
+      $message->{otherreports}{$bodypath} .=
+        MailScanner::Config::LanguageValue($message, 'partialmessage') . "\n";
+      $message->{othertypes}{$bodypath} .= 'e';
+    }
     $message->{otherinfected}++;
     $counter++;
+    #$message->{entityreports}{$entity} .=
+    #  MailScanner::Config::LanguageValue($message, 'partialmessage') . "\n";
+    #$message->{entitytypes}{$entity} .= "e";
   }
 
   # Now try the same on all the parts
@@ -386,7 +433,9 @@ sub FindHTMLExploits {
   #$disposition = 'inline' unless $disposition;
   if ($type && $type =~ /text\/html/i &&
       #$disposition !~ /attachment/i &&
-      defined($entity->body) && defined($entity->bodyhandle->path)) {
+      defined($entity->bodyhandle) &&
+      defined($entity->body) &&
+      defined($entity->bodyhandle->path)) {
     $counter += SearchHTMLBody($message, $id, $entity->bodyhandle->path,
                                $allowiframes, $allowobjects, $allowforms,
                                $allowscripts, $allowwebbugs,
@@ -425,12 +474,15 @@ sub SearchHTMLBody {
   $fh = new FileHandle;
   if ($fh->open("$filename")) {
 
+    my $loggingtags = MailScanner::Config::Value('loghtmltags', $message);
+
     # Search the file
     my $inobject    = 0;
     my $iframefound = 0;
     my $formfound   = 0;
     my $scriptfound = 0;
     my $webbugfound = 0;
+    my $phishingfound = 0;
     my $codebasefound = 0;
     my $attach = $filename;
     $attach = $1 if $filename =~ /([^\/]+)$/; # Strip off the path
@@ -450,6 +502,9 @@ sub SearchHTMLBody {
       # Find the img tag start
       $webbugfound = 1 if /\<img/i;
 
+      # Find the link tag start
+      $phishingfound = 1 if /\<a/i;
+
       # Find the object tag start
       $inobject = 1 if /\<object/i;
 
@@ -466,11 +521,18 @@ sub SearchHTMLBody {
     $silentviruses = ' ' .  MailScanner::Config::Value('silentviruses',
                      $message) . ' ';
 
+    if ($phishingfound && MailScanner::Config::Value('findphishing', $message)){
+      # Log the <A>
+      MailScanner::Log::InfoLog("<A> tag found in message %s from %s",
+        $id, $message->{from}) if $loggingtags;
+      # Mark the message
+      $message->{tagstoconvert} .= 'phishing ';
+      #$message->{bodymodified}   = 1;
+    }
     if ($iframefound) {
       # Log the <IFrame>
-      MailScanner::Log::InfoLog("HTML IFrame tag found in message %s from %s",
-        $id, $message->{from})
-        if MailScanner::Config::Value('logiframetags', $message);
+      MailScanner::Log::NoticeLog("HTML-IFrame tag found in message %s from %s",
+        $id, $message->{from}) if $loggingtags;
       # Mark the message
       if ($allowiframes) {
         if ($stripdangerous) {
@@ -491,10 +553,9 @@ sub SearchHTMLBody {
       }
     }
     if ($formfound) {
-      ## Log the <IFrame>
-      #MailScanner::Log::InfoLog("HTML Form tag found in message %s from %s",
-      #  $id, $message->{from})
-      #  if MailScanner::Config::Value('logformtags', $message);
+      ## Log the <Form>
+      MailScanner::Log::NoticeLog("HTML-Form tag found in message %s from %s",
+        $id, $message->{from}) if $loggingtags;
       # Mark the message
       if ($allowforms) {
         if ($stripdangerous) {
@@ -516,9 +577,8 @@ sub SearchHTMLBody {
     }
     if ($scriptfound) {
       ## Log the <script>
-      #MailScanner::Log::InfoLog("HTML Form tag found in message %s from %s",
-      #  $id, $message->{from})
-      #  if MailScanner::Config::Value('logformtags', $message);
+      MailScanner::Log::NoticeLog("HTML-Script tag found in message %s from %s",
+        $id, $message->{from}) if $loggingtags;
       # Mark the message
       if ($allowscripts) {
         if ($stripdangerous) {
@@ -540,8 +600,8 @@ sub SearchHTMLBody {
     }
     if ($webbugfound) {
       ## Log the <img>
-      #MailScanner::Log::InfoLog("HTML Img tag found in message %s from %s",
-      #  $id, $message->{from});
+      MailScanner::Log::NoticeLog("HTML Img tag found in message %s from %s",
+        $id, $message->{from}) if $loggingtags;
       #  if MailScanner::Config::Value('logwebbugs', $message);
       # Mark the message
       if ($allowwebbugs) {
@@ -555,24 +615,25 @@ sub SearchHTMLBody {
       } elsif ($convertwebbugs) {
         #print STDERR "Web Bug converted\n";
         $message->{tagstoconvert} .= 'webbug ';
-        $message->{bodymodified}   = 1;
+        # Only mark it for rebuilding if we actually found a webbug
+        # as we shouldn't rebuild if it was an innocent image
+        #$message->{bodymodified}   = 1;
         #print STDERR "Going to disarm web bugs\n";
       } else {
         #print STDERR "Web Bug ignored\n";
-        #
-        # Don't mark potential web bugs as infections, they are just images
-        # until we analyse them in disinfection.
-        #
-        #$message->{otherreports}{"$attach"} .=
-        #  MailScanner::Config::LanguageValue($message, 'foundwebbug') . "\n";
-        #$message->{othertypes}{"$attach"}   .= "c";
-        #$message->{otherinfected}++;
-        #$message->{silent} = 1 if $silentviruses =~ / HTML-WebBug /i;
-        #$counter++;
+        # Web bugs neither allowed nor converted. So must be stopped.
+        $message->{otherreports}{"$attach"} .=
+          MailScanner::Config::LanguageValue($message, 'foundwebbug') . "\n";
+        $message->{othertypes}{"$attach"}   .= "c";
+        $message->{otherinfected}++;
+        $message->{silent} = 1 if $silentviruses =~ / HTML-WebBug /i;
+        $counter++;
         1;
       }
     }
     if ($codebasefound) {
+      MailScanner::Log::NoticeLog("HTML-Object tag found in message %s from %s",
+        $id, $message->{from}) if $loggingtags;
       if ($allowobjects) {
         if ($stripdangerous) {
           $message->{needsstripping} = 1;
