@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SweepOther.pm 4453 2008-05-21 20:04:02Z sysjkf $
+#   $Id: SweepOther.pm 4553 2008-09-26 09:21:19Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ use POSIX qw(:signal_h setsid); # For Solaris 9 SIG bug workaround
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4453 $, 10;
+$VERSION = substr q$Revision: 4553 $, 10;
 
 # Attributes are
 #
@@ -243,6 +243,7 @@ sub ScanBatch {
             $message->{nametypes}{$safename}   .= "f";
             $counter++;
             $message->{nameinfected}++;
+            $message->DeleteFile($safename);
           }
         }
 
@@ -255,12 +256,12 @@ sub ScanBatch {
         next unless $FilenameRules;
 
         #foreach $i (@$FilenameRules) {
-        #  print STDERR "FilenameRule: $i\n";
+        #  #print STDERR "FilenameRule: $i\n";
         #}
 
-        my($allowdeny, $regexp);
+        my($allowdeny, $regexp, $iregexp);
         for ($i=0; !$MatchFound && $i<scalar(@$FilenameRules); $i++) {
-          ($allowdeny, $regexp, $logtext, $usertext)
+          ($allowdeny, $regexp, $iregexp, $logtext, $usertext)
             = split(/\0/, $FilenameRules->[$i]);
 
           #print STDERR "Filename match $i: \"$allowdeny\" \"$regexp\" \"$attach\" \"$safename\" $logtext $usertext\n";
@@ -283,7 +284,10 @@ sub ScanBatch {
             $counter++;
             $message->{nameinfected}++;
             # Do we want to delete the attachment or store it?
-            $message->{deleteattach}{$safename} = 1 if $allowdeny =~ /delete/;
+            if ($allowdeny =~ /delete/) {
+              $message->{deleteattach}{$safename} = 1;
+              $message->DeleteFile($safename);
+            }
           } elsif ($allowdeny =~ /@/) {
           # It's email addresses so replace the recipients list.
           $message->DeleteAllRecipients();
@@ -574,6 +578,7 @@ sub CheckFileTypesRules {
           $message->{nametypes}{$safename}   .= "f";
           $counter++;
           $message->{nameinfected}++;
+          $message->DeleteFile($safename);
         }
       }
 
@@ -603,7 +608,10 @@ sub CheckFileTypesRules {
           $counter++;
           $message->{nameinfected}++;
           # Do we want to delete the attachment or store it?
-          $message->{deleteattach}{$safename} = 1 if $allowdeny =~ /delete/;
+          if ($allowdeny =~ /delete/) {
+            $message->{deleteattach}{$safename} = 1;
+            $message->DeleteFile($safename);
+          }
         } elsif ($allowdeny =~ /@/) {
           # It's email addresses so replace the recipients list.
           $message->DeleteAllRecipients();
@@ -706,6 +714,7 @@ sub CheckFileTypesRules {
           $message->{nametypes}{$safename}   .= "f";
           $counter++;
           $message->{nameinfected}++;
+          $message->DeleteFile($safename);
         }
       }
 
@@ -734,7 +743,10 @@ sub CheckFileTypesRules {
           $counter++;
           $message->{nameinfected}++;
           # Do we want to delete the attachment or store it?
-          $message->{deleteattach}{$safename} = 1 if $allowdeny =~ /delete/;
+          if ($allowdeny =~ /delete/) {
+            $message->{deleteattach}{$safename} = 1;
+            $message->DeleteFile($safename);
+          }
         } elsif ($allowdeny =~ /@/) {
           # It's email addresses so replace the recipients list.
           $message->DeleteAllRecipients();
